@@ -8,6 +8,7 @@
 
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <signal.h>
 #if HAVE_STRING_H
 #include <string.h>
@@ -41,13 +42,16 @@
 #include "mibgroup/struct.h"
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_iterator.h>
-#include "mib_module_includes.h"
 
 #ifdef USING_AGENTX_SUBAGENT_MODULE
 #include "agentx/subagent.h"
 #include "agentx/client.h"
 #endif
 
+netsnmp_feature_child_of(agent_index_all, libnetsnmpagent)
+
+netsnmp_feature_child_of(remove_index, agent_index_all)
+
         /*
          * Initial support for index allocation
          */
@@ -451,12 +455,16 @@ register_index(netsnmp_variable_list * varbind, int flags,
          * Release an allocated index,
          *   to allow it to be used elsewhere
          */
+netsnmp_feature_child_of(release_index,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_RELEASE_INDEX
 int
 release_index(netsnmp_variable_list * varbind)
 {
     return (unregister_index(varbind, TRUE, NULL));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_RELEASE_INDEX */
 
+#ifndef NETSNMP_FEATURE_REMOVE_REMOVE_INDEX
         /*
          * Completely remove an allocated index,
          *   due to errors in the registration process.
@@ -466,6 +474,7 @@ remove_index(netsnmp_variable_list * varbind, netsnmp_session * ss)
 {
     return (unregister_index(varbind, FALSE, ss));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REMOVE_INDEX */
 
 void
 unregister_index_by_session(netsnmp_session * ss)
@@ -518,9 +527,30 @@ unregister_index(netsnmp_variable_list * varbind, int remember,
 
     for (idxptr2 = idxptr; idxptr2 != NULL;
          prev_idx_ptr = idxptr2, idxptr2 = idxptr2->next_idx) {
-        i = SNMP_MIN(varbind->val_len, idxptr2->varbind->val_len);
-        res2 =
-            memcmp(varbind->val.string, idxptr2->varbind->val.string, i);
+        switch (varbind->type) {
+        case ASN_INTEGER:
+            res2 =
+                (*varbind->val.integer -
+                 *idxptr2->varbind->val.integer);
+            break;
+        case ASN_OCTET_STR:
+            i = SNMP_MIN(varbind->val_len,
+                         idxptr2->varbind->val_len);
+            res2 =
+                memcmp(varbind->val.string,
+                       idxptr2->varbind->val.string, i);
+            break;
+        case ASN_OBJECT_ID:
+            res2 =
+                snmp_oid_compare(varbind->val.objid,
+                                 varbind->val_len / sizeof(oid),
+                                 idxptr2->varbind->val.objid,
+                                 idxptr2->varbind->val_len /
+                                 sizeof(oid));
+            break;
+        default:
+            return INDEX_ERR_WRONG_TYPE;        /* wrong type */
+        }
         if (res2 <= 0)
             break;
     }
@@ -568,6 +598,8 @@ unregister_index(netsnmp_variable_list * varbind, int remember,
     return SNMP_ERR_NOERROR;
 }
 
+netsnmp_feature_child_of(unregister_indexes,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_UNREGISTER_INDEXES
 int
 unregister_string_index(oid * name, size_t name_len, char *cp)
 {
@@ -607,6 +639,7 @@ unregister_oid_index(oid * name, size_t name_len,
                        value_len * sizeof(oid));
     return (unregister_index(&varbind, FALSE, main_session));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_UNREGISTER_INDEXES */
 
 void
 dump_idx_registry(void)
@@ -672,6 +705,8 @@ dump_idx_registry(void)
     }
 }
 
+netsnmp_feature_child_of(count_indexes, netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_UNUSED
 unsigned long
 count_indexes(oid * name, size_t namelen, int include_unallocated)
 {
@@ -691,7 +726,7 @@ count_indexes(oid * name, size_t namelen, int include_unallocated)
     }
     return n;
 }
-
+#endif /* NETSNMP_FEATURE_REMOVE_UNUSED */
 
 #ifdef TESTING
 netsnmp_variable_list varbind;
