@@ -9,6 +9,7 @@
  * distributed with the Net-SNMP package.
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #if HAVE_STRING_H
 #include <string.h>
@@ -21,6 +22,11 @@
 
 #include <net-snmp/agent/cache_handler.h>
 
+netsnmp_feature_child_of(cache_handler, mib_helpers)
+
+netsnmp_feature_child_of(cache_find_by_oid, cache_handler)
+netsnmp_feature_child_of(cache_get_head, cache_handler)
+
 static netsnmp_cache  *cache_head = NULL;
 static int             cache_outstanding_valid = 0;
 static int             _cache_load( netsnmp_cache *cache );
@@ -46,7 +52,7 @@ void            release_cached_resources(unsigned int regNo,
  *  expired caches, and will call the free_cache function for any expired
  *  cache.
  *
- *  The load_cache route should return a negative number if the cache
+ *  The load_cache routine should return a negative number if the cache
  *  was not successfully loaded. 0 or any positive number indicates successs.
  *
  *
@@ -79,6 +85,14 @@ void            release_cached_resources(unsigned int regNo,
  *  the cache when it expires. This is useful for keeping the cache fresh,
  *  even in the absence of incoming snmp requests.
  *
+ *  If NETSNMP_CACHE_RESET_TIMER_ON_USE is set, the expiry timer will be
+ *  reset on each cache access. In practice the 'timeout' becomes a timer
+ *  which triggers when the cache is no longer needed. This is useful
+ *  if the cache is automatically kept synchronized: e.g. by receiving
+ *  change notifications from Netlink, inotify or similar. This should
+ *  not be used if cache is not synchronized automatically as it would
+ *  result in stale cache information when if polling happens too fast.
+ *
  *
  *  Here are some suggestions for some common situations.
  *
@@ -110,12 +124,20 @@ void            release_cached_resources(unsigned int regNo,
  *          NETSNMP_CACHE_DONT_AUTO_RELEASE
  *          NETSNMP_CACHE_AUTO_RELOAD
  *
+ *  Dynamically updated, unloaded after timeout:
+ *      If the cache is kept up to date dynamically by listening for
+ *      change notifications somehow, but it should not be in memory
+ *      if it's not needed. Set the following flag:
+ *
+ *          NETSNMP_CACHE_RESET_TIMER_ON_USE
+ *
  *  @{
  */
 
 static void
 _cache_free( netsnmp_cache *cache );
 
+#ifndef NETSNMP_FEATURE_REMOVE_CACHE_GET_HEAD
 /** get cache head
  * @internal
  * unadvertised function to get cache head. You really should not
@@ -126,7 +148,9 @@ netsnmp_cache_get_head(void)
 {
     return cache_head;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_CACHE_GET_HEAD */
 
+#ifndef NETSNMP_FEATURE_REMOVE_CACHE_FIND_BY_OID
 /** find existing cache
  */
 netsnmp_cache *
@@ -142,6 +166,7 @@ netsnmp_cache_find_by_oid(const oid * rootoid, int rootoid_len)
     
     return NULL;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_CACHE_FIND_BY_OID */
 
 /** returns a cache
  */
@@ -186,6 +211,22 @@ netsnmp_cache_create(int timeout, NetsnmpCacheLoad * load_hook,
     return cache;
 }
 
+static netsnmp_cache *
+netsnmp_cache_ref(netsnmp_cache *cache)
+{
+    cache->refcnt++;
+    return cache;
+}
+
+static void
+netsnmp_cache_deref(netsnmp_cache *cache)
+{
+    if (--cache->refcnt == 0) {
+        netsnmp_cache_remove(cache);
+        netsnmp_cache_free(cache);
+    }
+}
+
 /** frees a cache
  */
 int
@@ -205,8 +246,8 @@ netsnmp_cache_free(netsnmp_cache *cache)
             sprint_realloc_objid((u_char **) &buf, &buf_len, &out_len,
                                  1, pos->rootoid, pos->rootoid_len);
             snmp_log(LOG_WARNING,
-		     "not freeing cache with root OID %s (still in list)\n",
-		     buf);
+                     "not freeing cache with root OID %s (still in list)\n",
+                     buf);
             free(buf);
             return SNMP_ERR_GENERR;
         }
@@ -218,6 +259,9 @@ netsnmp_cache_free(netsnmp_cache *cache)
     if (cache->valid)
         _cache_free(cache);
 
+    if (cache->timestampM)
+	free(cache->timestampM);
+
     if (cache->rootoid)
         free(cache->rootoid);
 
@@ -353,6 +397,17 @@ netsnmp_cache_handler_get(netsnmp_cache* cache)
     return ret;
 }
 
+/** Makes sure that memory allocated for the cache is freed when the handler
+ *  is unregistered.
+ */
+void netsnmp_cache_handler_owns_cache(netsnmp_mib_handler *handler)
+{
+    netsnmp_assert(handler->myvoid);
+    ((netsnmp_cache *)handler->myvoid)->refcnt++;
+    handler->data_clone = (void *(*)(void *))netsnmp_cache_ref;
+    handler->data_free = (void(*)(void*))netsnmp_cache_deref;
+}
+
 /** returns a cache handler that can be injected into a given handler chain.  
  */
 netsnmp_mib_handler *
@@ -368,12 +423,15 @@ netsnmp_get_cache_handler(int timeout, NetsnmpCacheLoad * load_hook,
         cache = netsnmp_cache_create(timeout, load_hook, free_hook,
                                      rootoid, rootoid_len);
         ret->myvoid = (void *) cache;
+        netsnmp_cache_handler_owns_cache(ret);
     }
     return ret;
 }
 
 /** functionally the same as calling netsnmp_register_handler() but also
  * injects a cache handler at the same time for you. */
+netsnmp_feature_child_of(netsnmp_cache_handler_register,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_NETSNMP_CACHE_HANDLER_REGISTER
 int
 netsnmp_cache_handler_register(netsnmp_handler_registration * reginfo,
                                netsnmp_cache* cache)
@@ -384,9 +442,12 @@ netsnmp_cache_handler_register(netsnmp_handler_registration * reginfo,
     netsnmp_inject_handler(reginfo, handler);
     return netsnmp_register_handler(reginfo);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_NETSNMP_CACHE_HANDLER_REGISTER */
 
 /** functionally the same as calling netsnmp_register_handler() but also
  * injects a cache handler at the same time for you. */
+netsnmp_feature_child_of(netsnmp_register_cache_handler,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_NETSNMP_REGISTER_CACHE_HANDLER
 int
 netsnmp_register_cache_handler(netsnmp_handler_registration * reginfo,
                                int timeout, NetsnmpCacheLoad * load_hook,
@@ -400,8 +461,9 @@ netsnmp_register_cache_handler(netsnmp_handler_registration * reginfo,
     netsnmp_inject_handler(reginfo, handler);
     return netsnmp_register_handler(reginfo);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_NETSNMP_REGISTER_CACHE_HANDLER */
 
-NETSNMP_STATIC_INLINE char *
+static char *
 _build_cache_name(const char *name)
 {
     char *dup = (char*)malloc(strlen(name) + strlen(CACHE_NAME) + 2);
@@ -441,11 +503,14 @@ netsnmp_cache_reqinfo_extract(netsnmp_agent_request_info * reqinfo,
 }
 
 /** Extract the cache information for a given request (PDU) */
+netsnmp_feature_child_of(netsnmp_extract_cache_info,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_NETSNMP_EXTRACT_CACHE_INFO
 netsnmp_cache  *
 netsnmp_extract_cache_info(netsnmp_agent_request_info * reqinfo)
 {
     return netsnmp_cache_reqinfo_extract(reqinfo, CACHE_NAME);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_NETSNMP_EXTRACT_CACHE_INFO */
 
 
 /** Check if the cache timeout has passed. Sets and return the expired flag. */
@@ -454,11 +519,13 @@ netsnmp_cache_check_expired(netsnmp_cache *cache)
 {
     if(NULL == cache)
         return 0;
-    
-    if(!cache->valid || (NULL == cache->timestamp) || (-1 == cache->timeout))
+    if (cache->expired)
+        return 1;
+    if(!cache->valid || (NULL == cache->timestampM) || (-1 == cache->timeout))
         cache->expired = 1;
     else
-        cache->expired = atime_ready(cache->timestamp, 1000 * cache->timeout);
+        cache->expired = netsnmp_ready_monotonic(cache->timestampM,
+                                                 1000 * cache->timeout);
     
     return cache->expired;
 }
@@ -492,11 +559,14 @@ netsnmp_cache_is_valid(netsnmp_agent_request_info * reqinfo,
 /** Is the cache valid for a given request?
  * for backwards compatability. netsnmp_cache_is_valid() is preferred.
  */
+netsnmp_feature_child_of(netsnmp_is_cache_valid,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_NETSNMP_IS_CACHE_VALID
 int
 netsnmp_is_cache_valid(netsnmp_agent_request_info * reqinfo)
 {
     return netsnmp_cache_is_valid(reqinfo, CACHE_NAME);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_NETSNMP_IS_CACHE_VALID */
 
 /** Implements the cache handler */
 int
@@ -526,7 +596,7 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
                    "cache not found, disabled or had no load method\n"));
         return SNMP_ERR_NOERROR;
     }
-    snprintf(addrstr,sizeof(addrstr), "%ld", cache);
+    snprintf(addrstr,sizeof(addrstr), "%ld", (long int)cache);
     DEBUGMSGTL(("helper:cache_handler", "using cache %s: ", addrstr));
     DEBUGMSGOID(("helper:cache_handler", cache->rootoid, cache->rootoid_len));
     DEBUGMSG(("helper:cache_handler", "\n"));
@@ -546,7 +616,9 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
     case MODE_GET:
     case MODE_GETNEXT:
     case MODE_GETBULK:
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */
 
         /*
          * only touch cache once per pdu request, to prevent a cache
@@ -568,6 +640,7 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
         /** next handler called automatically - 'AUTO_NEXT' */
         break;
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE2:
     case MODE_SET_FREE:
     case MODE_SET_ACTION:
@@ -589,6 +662,7 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
         }
         /** next handler called automatically - 'AUTO_NEXT' */
         break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
     default:
         snmp_log(LOG_WARNING, "cache_handler: Unrecognised mode (%d)\n",
@@ -596,6 +670,8 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
         netsnmp_request_set_error_all(requests, SNMP_ERR_GENERR);
         return SNMP_ERR_GENERR;
     }
+    if (cache->flags & NETSNMP_CACHE_RESET_TIMER_ON_USE)
+        netsnmp_set_monotonic_marker(&cache->timestampM);
     return SNMP_ERR_NOERROR;
 }
 
@@ -640,10 +716,7 @@ _cache_load( netsnmp_cache *cache )
                             0, release_cached_resources, NULL);
         cache_outstanding_valid = 1;
     }
-    if (cache->timestamp)
-        atime_setMarker(cache->timestamp);
-    else
-        cache->timestamp = atime_newMarker();
+    netsnmp_set_monotonic_marker(&cache->timestampM);
     DEBUGMSGT(("helper:cache_handler", " loaded (%d)\n", cache->timeout));
 
     return ret;
