@@ -1,4 +1,5 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -19,6 +20,19 @@ static void free_wrapper(void * p)
 #define free_wrapper free
 #endif
 
+netsnmp_feature_provide(watcher_all)
+netsnmp_feature_child_of(watcher_all, mib_helpers)
+netsnmp_feature_child_of(watcher_create_info6, watcher_all)
+netsnmp_feature_child_of(watcher_register_timestamp, watcher_all)
+netsnmp_feature_child_of(watcher_ulong_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_read_only_ulong_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_read_only_int_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_long_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_read_only_long_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_int_scalar, watcher_all)
+netsnmp_feature_child_of(read_only_counter32_scalar, watcher_all)
+netsnmp_feature_child_of(watcher_spinlock, watcher_all)
+
 /** @defgroup watcher watcher
  *  Watch a specified variable and process it as an instance or scalar object
  *  @ingroup leaf
@@ -51,6 +65,7 @@ netsnmp_init_watcher_info6(netsnmp_watcher_info *winfo,
     return winfo;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_CREATE_INFO6
 netsnmp_watcher_info *
 netsnmp_create_watcher_info6(void *data, size_t size, u_char type,
                              int flags, size_t max_size, size_t* size_p)
@@ -61,6 +76,7 @@ netsnmp_create_watcher_info6(void *data, size_t size, u_char type,
                                    size_p);
     return winfo;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_CREATE_INFO6 */
 
 netsnmp_watcher_info *
 netsnmp_init_watcher_info(netsnmp_watcher_info *winfo,
@@ -82,6 +98,11 @@ netsnmp_create_watcher_info(void *data, size_t size, u_char type, int flags)
     return winfo;
 }
 
+/**
+ * Register a watched scalar. The caller remains the owner of watchinfo.
+ *
+ * @see netsnmp_register_watched_instance2()
+ */
 int
 netsnmp_register_watched_instance(netsnmp_handler_registration *reginfo,
                                   netsnmp_watcher_info         *watchinfo)
@@ -95,6 +116,30 @@ netsnmp_register_watched_instance(netsnmp_handler_registration *reginfo,
     return netsnmp_register_instance(reginfo);
 }
 
+/**
+ * Register a watched scalar. Ownership of watchinfo is transferred to the handler.
+ *
+ * @see netsnmp_register_watched_instance()
+ */
+int
+netsnmp_register_watched_instance2(netsnmp_handler_registration *reginfo,
+				   netsnmp_watcher_info         *watchinfo)
+{
+    netsnmp_mib_handler *whandler;
+
+    whandler         = netsnmp_get_watcher_handler();
+    whandler->myvoid = (void *)watchinfo;
+    netsnmp_owns_watcher_info(whandler);
+
+    netsnmp_inject_handler(reginfo, whandler);
+    return netsnmp_register_instance(reginfo);
+}
+
+/**
+ * Register a watched scalar. The caller remains the owner of watchinfo.
+ *
+ * @see netsnmp_register_watched_scalar2()
+ */
 int
 netsnmp_register_watched_scalar(netsnmp_handler_registration *reginfo,
                                   netsnmp_watcher_info         *watchinfo)
@@ -108,6 +153,34 @@ netsnmp_register_watched_scalar(netsnmp_handler_registration *reginfo,
     return netsnmp_register_scalar(reginfo);
 }
 
+/**
+ * Register a watched scalar. Ownership of watchinfo is transferred to the handler.
+ *
+ * @see netsnmp_register_watched_scalar()
+ */
+int
+netsnmp_register_watched_scalar2(netsnmp_handler_registration *reginfo,
+                                  netsnmp_watcher_info         *watchinfo)
+{
+    netsnmp_mib_handler *whandler;
+
+    whandler         = netsnmp_get_watcher_handler();
+    whandler->myvoid = (void *)watchinfo;
+    netsnmp_owns_watcher_info(whandler);
+
+    netsnmp_inject_handler(reginfo, whandler);
+    return netsnmp_register_scalar(reginfo);
+}
+
+void
+netsnmp_owns_watcher_info(netsnmp_mib_handler *handler)
+{
+    netsnmp_assert(handler);
+    netsnmp_assert(handler->myvoid);
+    handler->data_clone = (void *(*)(void *))netsnmp_clone_watcher_info;
+    handler->data_free = free;
+}
+
 /** @cond */
 
 NETSNMP_STATIC_INLINE size_t
@@ -170,20 +243,13 @@ netsnmp_watcher_helper_handler(netsnmp_mib_handler *handler,
 {
     netsnmp_watcher_info  *winfo = (netsnmp_watcher_info *) handler->myvoid;
     netsnmp_watcher_cache *old_data;
-    int                    cmp;
 
     DEBUGMSGTL(("helper:watcher", "Got request:  %d\n", reqinfo->mode));
-    cmp = snmp_oid_compare(requests->requestvb->name,
-                           requests->requestvb->name_length,
-                           reginfo->rootoid, reginfo->rootoid_len);
-
     DEBUGMSGTL(( "helper:watcher", "  oid:"));
     DEBUGMSGOID(("helper:watcher", requests->requestvb->name,
                                    requests->requestvb->name_length));
     DEBUGMSG((   "helper:watcher", "\n"));
 
-
-
     switch (reqinfo->mode) {
         /*
          * data requests 
@@ -198,6 +264,7 @@ netsnmp_watcher_helper_handler(netsnmp_mib_handler *handler,
         /*
          * SET requests.  Should only get here if registered RWRITE 
          */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
         if (requests->requestvb->type != winfo->type) {
             netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_WRONGTYPE);
@@ -253,6 +320,12 @@ netsnmp_watcher_helper_handler(netsnmp_mib_handler *handler,
 
     case MODE_SET_COMMIT:
         break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+
+    default:
+        snmp_log(LOG_ERR, "watcher handler called with an unknown mode: %d\n",
+                 reqinfo->mode);
+        return SNMP_ERR_GENERR;
 
     }
 
@@ -291,6 +364,7 @@ netsnmp_watched_timestamp_register(netsnmp_mib_handler *whandler,
     return netsnmp_register_scalar(reginfo);   /* XXX - or instance? */
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_REGISTER_TIMESTAMP
 int
 netsnmp_register_watched_timestamp(netsnmp_handler_registration *reginfo,
                                    marker_t timestamp)
@@ -301,6 +375,7 @@ netsnmp_register_watched_timestamp(netsnmp_handler_registration *reginfo,
 
     return netsnmp_watched_timestamp_register(whandler, reginfo, timestamp);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_REGISTER_TIMESTAMP */
 
 
 int
@@ -311,21 +386,14 @@ netsnmp_watched_timestamp_handler(netsnmp_mib_handler *handler,
 {
     marker_t timestamp = (marker_t) handler->myvoid;
     long     uptime;
-    int      cmp;
 
     DEBUGMSGTL(("helper:watcher:timestamp",
                                "Got request:  %d\n", reqinfo->mode));
-    cmp = snmp_oid_compare(requests->requestvb->name,
-                           requests->requestvb->name_length,
-                           reginfo->rootoid, reginfo->rootoid_len);
-
     DEBUGMSGTL(( "helper:watcher:timestamp", "  oid:"));
     DEBUGMSGOID(("helper:watcher:timestamp", requests->requestvb->name,
                                    requests->requestvb->name_length));
     DEBUGMSG((   "helper:watcher:timestamp", "\n"));
 
-
-
     switch (reqinfo->mode) {
         /*
          * data requests 
@@ -345,11 +413,13 @@ netsnmp_watched_timestamp_handler(netsnmp_mib_handler *handler,
          * Timestamps are inherently Read-Only,
          *  so don't need to support SET requests.
          */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
         netsnmp_set_request_error(reqinfo, requests,
                                   SNMP_ERR_NOTWRITABLE);
         handler->flags |= MIB_HANDLER_AUTO_NEXT_OVERRIDE_ONCE;
         return SNMP_ERR_NOTWRITABLE;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
     }
 
     /* next handler called automatically - 'AUTO_NEXT' */
@@ -363,6 +433,8 @@ netsnmp_watched_timestamp_handler(netsnmp_mib_handler *handler,
      *
      ***************************/
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_SPINLOCK
+
 netsnmp_mib_handler *
 netsnmp_get_watched_spinlock_handler(void)
 {
@@ -388,7 +460,7 @@ netsnmp_register_watched_spinlock(netsnmp_handler_registration *reginfo,
     winfo            = netsnmp_create_watcher_info((void *)spinlock,
 		           sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE);
     netsnmp_inject_handler(reginfo, whandler);
-    return netsnmp_register_watched_scalar(reginfo, winfo);
+    return netsnmp_register_watched_scalar2(reginfo, winfo);
 }
 
 
@@ -400,25 +472,19 @@ netsnmp_watched_spinlock_handler(netsnmp_mib_handler *handler,
 {
     int     *spinlock = (int *) handler->myvoid;
     netsnmp_request_info *request;
-    int      cmp;
 
     DEBUGMSGTL(("helper:watcher:spinlock",
                                "Got request:  %d\n", reqinfo->mode));
-    cmp = snmp_oid_compare(requests->requestvb->name,
-                           requests->requestvb->name_length,
-                           reginfo->rootoid, reginfo->rootoid_len);
-
     DEBUGMSGTL(( "helper:watcher:spinlock", "  oid:"));
     DEBUGMSGOID(("helper:watcher:spinlock", requests->requestvb->name,
                                    requests->requestvb->name_length));
     DEBUGMSG((   "helper:watcher:spinlock", "\n"));
 
-
-
     switch (reqinfo->mode) {
         /*
          * Ensure the assigned value matches the current one
          */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
         for (request=requests; request; request=request->next) {
             if (request->processed)
@@ -439,11 +505,13 @@ netsnmp_watched_spinlock_handler(netsnmp_mib_handler *handler,
     case MODE_SET_COMMIT:
 	(*spinlock)++;
 	break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
     }
 
     /* next handler called automatically - 'AUTO_NEXT' */
     return SNMP_ERR_NOERROR;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_SPINLOCK */
 
     /***************************
      *
@@ -452,118 +520,142 @@ netsnmp_watched_spinlock_handler(netsnmp_mib_handler *handler,
      *
      ***************************/
 
+netsnmp_watcher_info *
+netsnmp_clone_watcher_info(netsnmp_watcher_info *winfo)
+{
+    netsnmp_watcher_info *copy = malloc(sizeof(*copy));
+    if (copy)
+	*copy = *winfo;
+    return copy;
+}
+
+static int
+register_scalar_watcher(const char* name,
+                        const oid* reg_oid, size_t reg_oid_len,
+                        void *data, size_t size, u_char type,
+                        Netsnmp_Node_Handler * subhandler, int mode)
+{
+    netsnmp_handler_registration *reginfo = NULL;
+    netsnmp_mib_handler *whandler = NULL;
+    netsnmp_watcher_info* watchinfo =
+        netsnmp_create_watcher_info(data, size, type, WATCHER_FIXED_SIZE);
+    if (watchinfo)
+        whandler = netsnmp_get_watcher_handler();
+    if (watchinfo && whandler) {
+        whandler->myvoid = watchinfo;
+	netsnmp_owns_watcher_info(whandler);
+        reginfo =
+            netsnmp_create_handler_registration(
+                name, subhandler, reg_oid, reg_oid_len, mode);
+    }
+    if (watchinfo && whandler && reginfo) {
+        netsnmp_inject_handler(reginfo, whandler);
+        return netsnmp_register_scalar(reginfo);
+    }
+    if (whandler)
+        netsnmp_handler_free(whandler);
+    else if (watchinfo)
+        free(watchinfo);
+    return SNMP_ERR_RESOURCEUNAVAILABLE;
+}
+
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_ULONG_SCALAR
 int
 netsnmp_register_ulong_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               u_long * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RWRITE ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( u_long ),
-                   ASN_UNSIGNED, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( u_long ),
+        ASN_UNSIGNED, subhandler, HANDLER_CAN_RWRITE);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_ULONG_SCALAR */
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_ULONG_SCALAR
 int
 netsnmp_register_read_only_ulong_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               u_long * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RONLY ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( u_long ),
-                   ASN_UNSIGNED, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( u_long ),
+        ASN_UNSIGNED, subhandler, HANDLER_CAN_RONLY);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_ULONG_SCALAR */
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_LONG_SCALAR
 int
 netsnmp_register_long_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               long * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RWRITE ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( long ),
-                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( long ),
+        ASN_INTEGER, subhandler, HANDLER_CAN_RWRITE);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_LONG_SCALAR */
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_LONG_SCALAR
 int
 netsnmp_register_read_only_long_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               long * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RONLY ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( long ),
-                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( long ),
+        ASN_INTEGER, subhandler, HANDLER_CAN_RONLY);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_LONG_SCALAR */
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_INT_SCALAR
 int
 netsnmp_register_int_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               int * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RWRITE ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( int ),
-                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( int ),
+        ASN_INTEGER, subhandler, HANDLER_CAN_RWRITE);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_INT_SCALAR */
 
+#ifndef NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_INT_SCALAR
 int
 netsnmp_register_read_only_int_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               int * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RONLY ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( int ),
-                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( int ),
+        ASN_INTEGER, subhandler, HANDLER_CAN_RONLY);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_WATCHER_READ_ONLY_INT_SCALAR */
 
-
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_COUNTER32_SCALAR
 int
 netsnmp_register_read_only_counter32_scalar(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               u_long * it,
                               Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_scalar(
-               netsnmp_create_handler_registration(
-                   name, subhandler,
-                   reg_oid, reg_oid_len,
-                   HANDLER_CAN_RONLY ),
-               netsnmp_create_watcher_info(
-                   (void *)it, sizeof( u_long ),
-                   ASN_COUNTER, WATCHER_FIXED_SIZE ));
+    return register_scalar_watcher(
+        name, reg_oid, reg_oid_len,
+        (void *)it, sizeof( u_long ),
+        ASN_COUNTER, subhandler, HANDLER_CAN_RONLY);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_COUNTER32_SCALAR */
 /**  @} */
 
