@@ -7,11 +7,18 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include "disman/event/mteTrigger.h"
 #include "disman/event/mteTriggerThresholdTable.h"
 
+netsnmp_feature_require(table_tdata)
+#ifndef NETSNMP_NO_WRITE_SUPPORT
+netsnmp_feature_require(check_vb_type_and_max_size)
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+
+static netsnmp_table_registration_info *table_info;
 
 /** Initializes the mteTriggerThresholdTable module */
 void
@@ -20,7 +27,6 @@ init_mteTriggerThresholdTable(void)
     static oid mteTThreshTable_oid[]   = { 1, 3, 6, 1, 2, 1, 88, 1, 2, 6 };
     size_t     mteTThreshTable_oid_len = OID_LENGTH(mteTThreshTable_oid);
     netsnmp_handler_registration    *reg;
-    netsnmp_table_registration_info *table_info;
 
     /*
      * Ensure the (combined) table container is available...
@@ -30,11 +36,19 @@ init_mteTriggerThresholdTable(void)
     /*
      * ... then set up the MIB interface to the mteTriggerThresholdTable slice
      */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     reg = netsnmp_create_handler_registration("mteTriggerThresholdTable",
                                             mteTriggerThresholdTable_handler,
                                             mteTThreshTable_oid,
                                             mteTThreshTable_oid_len,
                                             HANDLER_CAN_RWRITE);
+#else /* !NETSNMP_NO_WRITE_SUPPORT */
+    reg = netsnmp_create_handler_registration("mteTriggerThresholdTable",
+                                            mteTriggerThresholdTable_handler,
+                                            mteTThreshTable_oid,
+                                            mteTThreshTable_oid_len,
+                                            HANDLER_CAN_RONLY);
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */
 
     table_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_registration_info);
     netsnmp_table_helper_add_indexes(table_info,
@@ -51,6 +65,15 @@ init_mteTriggerThresholdTable(void)
     DEBUGMSGTL(("disman:event:init", "Trigger Threshold Table\n"));
 }
 
+void
+shutdown_mteTriggerThresholdTable(void)
+{
+    if (table_info) {
+	netsnmp_table_registration_info_free(table_info);
+	table_info = NULL;
+    }
+}
+
 
 /** handles requests for the mteTriggerThresholdTable table */
 int
@@ -74,6 +97,9 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
          */
     case MODE_GET:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct mteTrigger *) netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info(request);
 
@@ -82,8 +108,10 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
              *   rows where the mteTriggerTest 'threshold(2)' bit is set.
              * So skip entries where this isn't the case.
              */
-            if (!entry || !(entry->mteTriggerTest & MTE_TRIGGER_THRESHOLD ))
+            if (!entry || !(entry->mteTriggerTest & MTE_TRIGGER_THRESHOLD )) {
+                netsnmp_request_set_error(request, SNMP_NOSUCHINSTANCE);
                 continue;
+            }
 
             switch (tinfo->colnum) {
             case COLUMN_MTETRIGGERTHRESHOLDSTARTUP:
@@ -160,11 +188,15 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
         }
         break;
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
         /*
          * Write-support
          */
     case MODE_SET_RESERVE1:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct mteTrigger *) netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info(request);
 
@@ -252,6 +284,9 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
 
     case MODE_SET_ACTION:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct mteTrigger *) netsnmp_tdata_extract_entry(request);
             if (!entry) {
                 /*
@@ -272,6 +307,9 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
          *  (reasonably) safe to apply them in the Commit phase
          */
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct mteTrigger *) netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info(request);
 
@@ -344,6 +382,8 @@ mteTriggerThresholdTable_handler(netsnmp_mib_handler *handler,
             }
         }
         break;
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */
+
     }
     return SNMP_ERR_NOERROR;
 }
