@@ -7,12 +7,27 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include "utilities/iquery.h"
 #include "disman/schedule/schedCore.h"
 #include "disman/schedule/schedTable.h"
 
+netsnmp_feature_require(iquery)
+netsnmp_feature_require(iquery_pdu_session)
+netsnmp_feature_require(table_tdata)
+netsnmp_feature_require(date_n_time)
+netsnmp_feature_require(check_vb_uint)
+#ifndef NETSNMP_NO_WRITE_SUPPORT
+netsnmp_feature_require(check_vb_type_and_max_size)
+netsnmp_feature_require(check_vb_oid)
+netsnmp_feature_require(check_vb_truthvalue)
+netsnmp_feature_require(table_tdata_insert_row)
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+
+static netsnmp_table_registration_info *table_info;
+
 /** Initializes the schedTable module */
 void
 init_schedTable(void)
@@ -20,7 +35,6 @@ init_schedTable(void)
     static oid      schedTable_oid[] = { 1, 3, 6, 1, 2, 1, 63, 1, 2 };
     size_t          schedTable_oid_len = OID_LENGTH(schedTable_oid);
     netsnmp_handler_registration    *reg;
-    netsnmp_table_registration_info *table_info;
 
     DEBUGMSGTL(("disman:schedule:init", "Initializing table\n"));
     /*
@@ -48,6 +62,14 @@ init_schedTable(void)
     netsnmp_tdata_register(reg, schedule_table, table_info);
 }
 
+void
+shutdown_schedTable(void)
+{
+    if (table_info) {
+	netsnmp_table_registration_info_free(table_info);
+	table_info = NULL;
+    }
+}
 
 /** handles requests for the schedTable table */
 int
@@ -76,6 +98,9 @@ schedTable_handler(netsnmp_mib_handler *handler,
          */
     case MODE_GET:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct schedTable_entry *)
                     netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info( request);
@@ -188,6 +213,9 @@ schedTable_handler(netsnmp_mib_handler *handler,
          */
     case MODE_SET_RESERVE1:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct schedTable_entry *)
                     netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info( request);
@@ -318,6 +346,9 @@ schedTable_handler(netsnmp_mib_handler *handler,
 
     case MODE_SET_RESERVE2:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             tinfo = netsnmp_extract_table_info(request);
 
             switch (tinfo->colnum) {
@@ -348,6 +379,9 @@ schedTable_handler(netsnmp_mib_handler *handler,
 
     case MODE_SET_FREE:
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             tinfo = netsnmp_extract_table_info(request);
 
             switch (tinfo->colnum) {
@@ -395,7 +429,11 @@ schedTable_handler(netsnmp_mib_handler *handler,
          * All these assignments are "unfailable", so it's
          *  (reasonably) safe to apply them in the Commit phase
          */
+        entry = NULL;
         for (request = requests; request; request = request->next) {
+            if (request->processed)
+                continue;
+
             entry = (struct schedTable_entry *)
                     netsnmp_tdata_extract_entry(request);
             tinfo = netsnmp_extract_table_info( request);
@@ -438,7 +476,7 @@ schedTable_handler(netsnmp_mib_handler *handler,
                 recalculate = 1;
                 break;
             case COLUMN_SCHEDCONTEXTNAME:
-                memset(entry->schedContextName, 0, SCHED_STR1_LEN+1);
+                memset(entry->schedContextName, 0, sizeof(entry->schedContextName));
                 memcpy(entry->schedContextName,
                                            request->requestvb->val.string,
                                            request->requestvb->val_len);
@@ -492,8 +530,10 @@ schedTable_handler(netsnmp_mib_handler *handler,
                 break;
             }
         }
-        if (recalculate)
+        if (recalculate) {
+            netsnmp_assert(entry);
             sched_nextTime(entry);
+        }
         break;
     }
     return SNMP_ERR_NOERROR;
