@@ -39,8 +39,8 @@ netsnmp_swinst_arch_init(void)
 {
     strlcpy(pkg_directory, "/var/lib/dpkg/info", sizeof(pkg_directory));
     snprintf(apt_fmt, SNMP_MAXBUF, "%%%d[^#]#%%%d[^#]#%%%d[^#]#%%%d[^#]#%%%d[^#]#%%%ds",
-	SNMP_MAXBUF, SNMP_MAXBUF, SNMP_MAXBUF,
-	SNMP_MAXBUF, SNMP_MAXBUF, SNMP_MAXBUF);
+	SNMP_MAXBUF-1, SNMP_MAXBUF-1, SNMP_MAXBUF-1,
+	SNMP_MAXBUF-1, SNMP_MAXBUF-1, SNMP_MAXBUF-1);
 }
 
 void
@@ -65,7 +65,6 @@ netsnmp_swinst_arch_load( netsnmp_container *container, u_int flags)
     char buf[BUFSIZ];
     netsnmp_swinst_entry *entry;
     int i = 0;
-    int rc;
 
     if (p == NULL) {
 	snmp_perror("dpkg-list");
@@ -77,7 +76,7 @@ netsnmp_swinst_arch_load( netsnmp_container *container, u_int flags)
         entry = netsnmp_swinst_entry_create( i++ );
         if (NULL == entry)
             continue;   /* error already logged by function */
-        rc = CONTAINER_INSERT(container, entry);
+        CONTAINER_INSERT(container, entry);
 
 	sscanf(buf, apt_fmt, package, version, section, priority, essential, status);
 	if (strstr(status, "not-installed"))
@@ -96,7 +95,7 @@ netsnmp_swinst_arch_load( netsnmp_container *container, u_int flags)
     }
     pclose(p);
     DEBUGMSGTL(("swinst:load:arch"," loaded %d entries\n",
-                CONTAINER_SIZE(container)));
+                (int) CONTAINER_SIZE(container)));
 
     return 0;
 }
