@@ -120,7 +120,7 @@ netsnmp_arch_swrun_container_load( netsnmp_container *container, u_int flags)
          * skip empty names.
          * p_stat = (SIDL|SRUN|SSLEEP|SSTOP|SZOMB)
          */
-        if ((NULL == processes[i].kp_proc.p_comm) ||
+        if (('\0' == processes[i].kp_proc.p_comm[0]) ||
             (0 == processes[i].kp_proc.p_pid)) {
             DEBUGMSGTL(("swrun:load:arch",
                         " skipping p_comm '%s', pid %5d, p_pstat %d\n",
@@ -172,9 +172,10 @@ netsnmp_arch_swrun_container_load( netsnmp_container *container, u_int flags)
          * check for system processes
          */
         if (P_SYSTEM & processes[i].kp_proc.p_flag) {
-            entry->hrSWRunType = 2; /* operatingSystem */
+            entry->hrSWRunType = HRSWRUNTYPE_OPERATINGSYSTEM;
             DEBUGMSGTL(("swrun:load:arch", SWRUNINDENT "SYSTEM\n"));
         }
+        else entry->hrSWRunType = HRSWRUNTYPE_APPLICATION;
 
         /*
          * get mem size, run time
@@ -192,7 +193,7 @@ netsnmp_arch_swrun_container_load( netsnmp_container *container, u_int flags)
             } at, ns;
             at.uq = taskinfo.ptinfo.pti_total_user +
                     taskinfo.ptinfo.pti_total_system;
-            ns.uw = AbsoluteToNanoseconds( at.uw );
+            ns = at;
             ns.uq = ns.uq / 10000000LL; /* nano to deci */
             if (task_mem > INT32_MAX) {
                 DEBUGMSGTL(("swrun:load:arch", SWRUNINDENT "mem overflow\n"));
@@ -209,7 +210,7 @@ netsnmp_arch_swrun_container_load( netsnmp_container *container, u_int flags)
     free(processes);
 
     DEBUGMSGTL(("swrun:load:arch"," loaded %d entries\n",
-                CONTAINER_SIZE(container)));
+                (int)CONTAINER_SIZE(container)));
 
     return 0;
 }
@@ -385,8 +386,8 @@ _set_command_name(netsnmp_swrun_entry *entry)
             break; /* End of exec_path reached. */
     }
     if (cp != exec_path + len) {
-        DEBUGMSGTL(("swrun:load:arch:_cn"," OFF BY %d\n",
-                    (exec_path + len) - cp));
+        DEBUGMSGTL(("swrun:load:arch:_cn", " OFF BY %d\n",
+                    (int)((exec_path + len) - cp)));
         netsnmp_assert( cp == exec_path + len );
     }
 #endif
@@ -457,7 +458,7 @@ _set_command_name(netsnmp_swrun_entry *entry)
          */
         if(entry->hrSWRunParameters_len < sizeof(entry->hrSWRunParameters)-1) {
             strlcat(&entry->hrSWRunParameters[entry->hrSWRunParameters_len],
-                    argN, sizeof(entry->hrSWRunParameters));
+                    argN, sizeof(entry->hrSWRunParameters)-entry->hrSWRunParameters_len-1);
             entry->hrSWRunParameters_len = strlen(entry->hrSWRunParameters);
             if ((entry->hrSWRunParameters_len+2 < sizeof(entry->hrSWRunParameters)-1) && (0 != nargs)) {
                 /* add space between params */
