@@ -5,10 +5,25 @@
 #ifndef _MIBGROUP_ICMP_H
 #define _MIBGROUP_ICMP_H
 
-config_arch_require(solaris2,        kernel_sunos5)
+config_arch_require(solaris2,  kernel_sunos5)
 config_arch_require(linux,     mibII/kernel_linux)
+config_arch_require(freebsd4,  mibII/kernel_sysctl)
+config_arch_require(freebsd5,  mibII/kernel_sysctl)
+config_arch_require(freebsd6,  mibII/kernel_sysctl)
+config_arch_require(freebsd7,  mibII/kernel_sysctl)
+config_arch_require(freebsd8,  mibII/kernel_sysctl)
+config_arch_require(freebsd9,  mibII/kernel_sysctl)
+config_arch_require(freebsd10, mibII/kernel_sysctl)
 config_arch_require(netbsd,    mibII/kernel_netbsd)
 config_arch_require(netbsdelf, mibII/kernel_netbsd)
+config_arch_require(openbsd4,  mibII/kernel_sysctl)
+config_arch_require(openbsd5,  mibII/kernel_sysctl)
+config_arch_require(dragonfly2, mibII/kernel_sysctl)
+config_arch_require(dragonfly3, mibII/kernel_sysctl)
+config_arch_require(darwin10,  mibII/kernel_sysctl)
+config_arch_require(darwin11,  mibII/kernel_sysctl)
+config_arch_require(darwin12,  mibII/kernel_sysctl)
+config_arch_require(darwin13,  mibII/kernel_sysctl)
 
 #include <net-snmp/agent/cache_handler.h>
 
