@@ -15,6 +15,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #if defined(NETSNMP_IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
 #define _KERNEL 1
@@ -120,6 +121,8 @@
 #include <windows.h>
 #endif
 
+netsnmp_feature_require(interface_legacy)
+
         /*********************
 	 *
 	 *  Kernel & interface information,
@@ -871,7 +874,7 @@ var_ipAddrEntry(struct variable *vp,
     oid             current[14], *op;
     u_char         *cp;
     int             lowinterface = -1;
-    int             i, interface;
+    int             i;
     static in_addr_t	addr_ret;
     
     /*
@@ -978,6 +981,7 @@ var_ipAddrEntry(struct variable *vp,
     DWORD           status = NO_ERROR;
     DWORD           statusRetry = NO_ERROR;
     DWORD           dwActualSize = 0;
+    void           *result = NULL;
     static in_addr_t 	addr_ret;
     
     /*
@@ -1014,7 +1018,6 @@ var_ipAddrEntry(struct variable *vp,
                 }
             } else {
                 if (snmp_oid_compare(current, 14, name, *length) > 0) {
-
                     lowinterface = i;
                     memcpy(lowest, current, 14 * sizeof(oid));
                     break;      /* Since the table is sorted, no need to search further  */
@@ -1023,46 +1026,52 @@ var_ipAddrEntry(struct variable *vp,
         }
     }
 
-    if (lowinterface < 0) {
-        free(pIpAddrTable);
-        return NULL;
-    }
+    if (lowinterface < 0)
+        goto out;
+
     i = lowinterface;
     memcpy(name, lowest, 14 * sizeof(oid));
     *length = 14;
     *write_method = 0;
-    *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
-    	 *var_len = sizeof(addr_ret);
-    	 addr_ret = pIpAddrTable->table[i].dwAddr;
-        return (u_char *) & addr_ret;
+        *var_len = sizeof(addr_ret);
+        addr_ret = pIpAddrTable->table[i].dwAddr;
+        result = &addr_ret;
+        break;
 
     case IPADIFINDEX:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwIndex;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     case IPADNETMASK:
-    	 *var_len = sizeof(addr_ret);
+        *var_len = sizeof(addr_ret);
         addr_ret = pIpAddrTable->table[i].dwMask;
-        free(pIpAddrTable);
-        return (u_char *) & addr_ret;
+        result = &addr_ret;
+        break;
 
     case IPADBCASTADDR:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwBCastAddr;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     case IPADREASMMAX:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwReasmSize;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     default:
         DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n",
                     vp->magic));
+        break;
     }
-    return NULL;
+
+out:
+    free(pIpAddrTable);
+    return result;
 }
 #endif                          /* WIN32 cygwin */
