@@ -95,7 +95,6 @@
 #include <time.h>
 #include <windows.h>
 #include "../../win32/Snmp-winExtDLL.h"
-#include "../../win32/MgmtApi-winExtDLL.h"
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/library/snmp_assert.h>
@@ -156,6 +155,9 @@ typedef         BOOL(WINAPI * PFNSNMPEXTENSIONTRAP) (AsnObjectIdentifier *
 
 typedef         VOID(WINAPI * PFNSNMPEXTENSIONCLOSE) (void);
 
+typedef BOOL (WINAPI *pfIsWow64Process)(HANDLE hProcess, BOOL *Wow64Process);
+
+
 /**
  * Extensible array, a data structure similar to the C++ STL class
  * std::vector<>.
@@ -179,6 +181,7 @@ typedef struct {
     HANDLE          dll_handle;                      /**< DLL handle. */
     PFNSNMPEXTENSIONINIT pfSnmpExtensionInit;
     PFNSNMPEXTENSIONINITEX pfSnmpExtensionInitEx;
+    PFNSNMPEXTENSIONCLOSE pfSnmpExtensionClose;
     PFNSNMPEXTENSIONQUERY pfSnmpExtensionQuery;
     PFNSNMPEXTENSIONQUERYEX pfSnmpExtensionQueryEx;
     PFNSNMPEXTENSIONTRAP pfSnmpExtensionTrap;
@@ -286,6 +289,8 @@ static void    *xarray_reserve(xarray * a, int reserved);
 #define WINEXTDLL_VIEW(i)       ((winextdll_view*)s_winextdll_view.p)[i]
 #define TRAPEVENT(i)            ((HANDLE*)s_trapevent.p)[i]
 #define TRAPEVENT_TO_DLLINFO(i) ((winextdll**)s_trapevent_to_dllinfo.p)[i]
+static const oid mibii_system_mib[] = { 1, 3, 6, 1, 2, 1, 1 };
+static OSVERSIONINFO s_versioninfo = { sizeof(s_versioninfo) };
 static xarray   s_winextdll = { 0, sizeof(winextdll) };
 static xarray   s_winextdll_view = { 0, sizeof(winextdll_view) };
 static xarray   s_trapevent = { 0, sizeof(HANDLE) };
@@ -301,11 +306,21 @@ static context_info *context_info_head;
 void
 init_winExtDLL(void)
 {
-    BOOL            result;
+    BOOL            result, is_wow64_process = FALSE;
     int             i;
+    uint32_t        uptime_reference;
+    pfIsWow64Process IsWow64Process;
 
     DEBUGMSG(("winExtDLL", "init_winExtDLL started.\n"));
 
+    GetVersionEx(&s_versioninfo);
+
+    IsWow64Process =
+      (pfIsWow64Process)GetProcAddress(GetModuleHandle("kernel32"),
+                                       "IsWow64Process");
+    if (IsWow64Process)
+        (*IsWow64Process)(GetCurrentProcess(), &is_wow64_process);
+
     SnmpSvcInitUptime();
 
     read_extension_dlls_from_registry();
@@ -367,6 +382,8 @@ init_winExtDLL(void)
         ext_dll_info->pfSnmpExtensionInitEx = (PFNSNMPEXTENSIONINITEX)
             GetProcAddress(ext_dll_info->dll_handle,
                            "SnmpExtensionInitEx");
+        ext_dll_info->pfSnmpExtensionClose = (PFNSNMPEXTENSIONCLOSE)
+            GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionClose");
         ext_dll_info->pfSnmpExtensionQuery = (PFNSNMPEXTENSIONQUERY)
             GetProcAddress(ext_dll_info->dll_handle, "SnmpExtensionQuery");
         ext_dll_info->pfSnmpExtensionQueryEx = (PFNSNMPEXTENSIONQUERYEX)
@@ -383,30 +400,61 @@ init_winExtDLL(void)
                      ext_dll_info->dll_name);
         }
 
+        /*
+         * At least on a 64-bit Windows 7 system invoking SnmpExtensionInit()
+         * in the 32-bit version of evntagnt.dll hangs. Also, all queries in
+         * lmmib2.dll fail with "generic error" on a 64-bit Windows 7 system.
+         * So skip these two DLLs.
+         */
+        if (s_versioninfo.dwMajorVersion >= 6
+            && ((is_wow64_process
+                 && basename_equals(ext_dll_info->dll_name, "evntagnt.dll"))
+                || basename_equals(ext_dll_info->dll_name, "lmmib2.dll"))) {
+            DEBUGMSG(("winExtDLL", "init_winExtDLL: skipped DLL %s.\n",
+                      ext_dll_info->dll_name));
+            continue;
+        }
+
         /*
          * Init and get first supported view from Windows SNMP extension DLL.
          * Note: although according to the documentation of SnmpExtensionInit()
          * the first argument of this function should be ignored by extension
-         * DLLs, passing the value GetTickCount() / 10 is necessary to make
-         * inetmib1.dll work correctly. Passing zero as the first argument
-         * causes inetmib1.dll to report an incorrect value for sysUpTime.0
-         * and also causes the same DLL not to send linkUp or linkDown traps.
+         * DLLs, passing a correct value for this first argument is necessary
+         * to make inetmib1.dll work correctly. Passing zero as the first
+         * argument causes inetmib1.dll to report an incorrect value for
+         * sysUpTime.0 and also causes the same DLL not to send linkUp or
+         * linkDown traps.
          */
         ext_dll_info->subagentTrapEvent = NULL;
         view.idLength = 0;
-        view.ids = 0;
+        view.ids = NULL;
+        if (!is_wow64_process && s_versioninfo.dwMajorVersion >= 6)
+            uptime_reference = GetTickCount() - 10 * SnmpSvcGetUptime();
+        else
+            uptime_reference = GetTickCount() / 10;
         result =
-            ext_dll_info->pfSnmpExtensionInit(GetTickCount() / 10,
+            ext_dll_info->pfSnmpExtensionInit(uptime_reference,
                                               &ext_dll_info->
                                               subagentTrapEvent, &view);
 
         if (!result) {
-            snmp_log(LOG_ERR,
-                     "init_winExtDLL: initialization of DLL %s failed.\n",
-                     ext_dll_info->dll_name);
-            FreeLibrary(ext_dll_info->dll_handle);
-            ext_dll_info->dll_handle = 0;
-            continue;
+            DEBUGMSG(("winExtDLL",
+                      "init_winExtDLL: initialization of DLL %s failed.\n",
+                      ext_dll_info->dll_name));
+            /*
+             * At least on Windows 7 SnmpExtensionInit() in some extension
+             * agent DLLs returns "FALSE" although initialization
+             * succeeded. Hence ignore the SnmpExtensionInit() return value on
+             * Windows Vista and later.
+             */
+            if (s_versioninfo.dwMajorVersion < 6) {
+                snmp_log(LOG_ERR,
+                         "init_winExtDLL: initialization of DLL %s failed.\n",
+                         ext_dll_info->dll_name);
+                FreeLibrary(ext_dll_info->dll_handle);
+                ext_dll_info->dll_handle = 0;
+                continue;
+            }
         }
 
         if (ext_dll_info->subagentTrapEvent != NULL) {
@@ -417,6 +465,26 @@ init_winExtDLL(void)
 
         memset(&ext_dll_view_info, 0, sizeof(ext_dll_view_info));
         ext_dll_view_info.winextdll_info = ext_dll_info;
+        if (view.idLength == 0) {
+            DEBUGMSG(("winExtDLL",
+                      "init_winExtDLL: DLL %s did not register an OID range.\n",
+                      ext_dll_info->dll_name));
+            continue;
+        }
+        /*
+         * Skip the mib-2 system section on Windows Vista and later because
+         * at least on a 64-bit Windows 7 system all queries in that section
+         * fail with status "generic error".
+         */
+        if (s_versioninfo.dwMajorVersion >= 6
+            && snmp_oid_compare_w_n(view.ids, view.idLength, mibii_system_mib,
+                                    sizeof(mibii_system_mib) /
+                                    sizeof(mibii_system_mib[0])) == 0) {
+            DEBUGMSG(("winExtDLL",
+                      "init_winExtDLL: skipping system section of DLL %s.\n",
+                      ext_dll_info->dll_name));
+            continue;
+        }
         copy_oid_n_w(ext_dll_view_info.name, &ext_dll_view_info.name_length,
                      view.ids, view.idLength);
         xarray_push_back(&s_winextdll_view, &ext_dll_view_info);
@@ -485,16 +553,22 @@ shutdown_winExtDLL(void)
 
     for (i = s_winextdll.size - 1; i >= 0; i--) {
         winextdll      *const ext_dll_info = &WINEXTDLL(i);
-        /*
-         * Freeing the Broadcom SNMP extension libraries triggers a deadlock,
-         * so skip bcmif.dll and baspmgnt.dll. 
-         */
-        if (ext_dll_info->dll_handle != 0
-            && !basename_equals(ext_dll_info->dll_name, "bcmif.dll")
-            && !basename_equals(ext_dll_info->dll_name, "baspmgnt.dll")) {
-            DEBUGMSG(("winExtDLL", "unloading %s.\n",
-                      ext_dll_info->dll_name));
-            FreeLibrary(ext_dll_info->dll_handle);
+        if (ext_dll_info->dll_handle) {
+            if (ext_dll_info->pfSnmpExtensionClose) {
+                DEBUGMSG(("winExtDLL", "closing %s.\n",
+                          ext_dll_info->dll_name));
+                ext_dll_info->pfSnmpExtensionClose();
+            }
+            /*
+             * Freeing the Broadcom SNMP extension libraries triggers
+             * a deadlock, so skip bcmif.dll and baspmgnt.dll.
+             */
+            if (!basename_equals(ext_dll_info->dll_name, "bcmif.dll")
+                && !basename_equals(ext_dll_info->dll_name, "baspmgnt.dll")) {
+                DEBUGMSG(("winExtDLL", "unloading %s.\n",
+                          ext_dll_info->dll_name));
+                FreeLibrary(ext_dll_info->dll_handle);
+            }
         }
         free(ext_dll_info->dll_name);
     }
@@ -524,7 +598,7 @@ basename_equals(const char *path, const char *basename)
 
     return path_len >= basename_len + 1
         && path[path_len - basename_len - 1] == '\\'
-        && stricmp(path + path_len - basename_len, basename) == 0;
+        && strcasecmp(path + path_len - basename_len, basename) == 0;
 }
 
 /**
@@ -682,6 +756,79 @@ get_context_info(const int index)
     return NULL;
 }
 
+/*
+ * Translate Net-SNMP request mode into an SnmpExtensionQuery() PDU type
+ * or into an SnmpExtensionQueryEx() request type.
+ */
+static int
+get_request_type(int mode, int request_type, UINT *nRequestType)
+{
+    switch (request_type) {
+    case 0:
+        /* SnmpExtensionQuery() PDU type */
+        switch (mode) {
+        case MODE_GET:
+            *nRequestType = SNMP_PDU_GET;
+            return 1;
+        case MODE_GETNEXT:
+            *nRequestType = SNMP_PDU_GETNEXT;
+            return 1;
+        case MODE_SET_RESERVE1:
+            return 0;
+        case MODE_SET_RESERVE2:
+            return 0;
+        case MODE_SET_ACTION:
+            return 0;
+        case MODE_SET_UNDO:
+            return 0;
+        case MODE_SET_COMMIT:
+            *nRequestType = SNMP_PDU_SET;
+            return 1;
+        case MODE_SET_FREE:
+            return 0;
+        default:
+            DEBUGMSG(("winExtDLL", "internal error: invalid mode %d.\n", mode));
+            netsnmp_assert(0);
+            return 0;
+        }
+    case 1:
+        /* SnmpExtensionQueryEx() request type */
+        switch (mode) {
+        case MODE_GET:
+            *nRequestType = SNMP_EXTENSION_GET;
+            return 1;
+        case MODE_GETNEXT:
+            *nRequestType = SNMP_EXTENSION_GET_NEXT;
+            return 1;
+        case MODE_SET_RESERVE1:
+            *nRequestType = SNMP_EXTENSION_SET_TEST;
+            return 1;
+        case MODE_SET_RESERVE2:
+            return 0;
+        case MODE_SET_ACTION:
+            return 0;
+        case MODE_SET_UNDO:
+            *nRequestType = SNMP_EXTENSION_SET_UNDO;
+            return 1;
+        case MODE_SET_COMMIT:
+            *nRequestType = SNMP_EXTENSION_SET_COMMIT;
+            return 1;
+        case MODE_SET_FREE:
+            *nRequestType = SNMP_EXTENSION_SET_CLEANUP;
+            return 1;
+        default:
+            DEBUGMSG(("winExtDLL", "internal error: invalid mode %d.\n", mode));
+            netsnmp_assert(0);
+            return 0;
+        }
+    default:
+        DEBUGMSG(("winExtDLL", "internal error: invalid argument %d.\n",
+                  request_type));
+        netsnmp_assert(0);
+        return 0;
+    }
+}
+
 static int
 var_winExtDLL(netsnmp_mib_handler *handler,
               netsnmp_handler_registration *reginfo,
@@ -708,35 +855,8 @@ var_winExtDLL(netsnmp_mib_handler *handler,
         return SNMP_ERR_GENERR;
     }
 
-    switch (reqinfo->mode) {
-    case MODE_GET:
-        nRequestType = SNMP_EXTENSION_GET;
-        netsnmp_assert(!context_info_head);
-        break;
-    case MODE_GETNEXT:
-        nRequestType = SNMP_EXTENSION_GET_NEXT;
-        netsnmp_assert(!context_info_head);
-        break;
-    case MODE_SET_RESERVE1:
-        nRequestType = SNMP_EXTENSION_SET_TEST;
-        break;
-    case MODE_SET_RESERVE2:
-        return SNMP_ERR_NOERROR;
-    case MODE_SET_ACTION:
-        return SNMP_ERR_NOERROR;
-    case MODE_SET_UNDO:
-        nRequestType = SNMP_EXTENSION_SET_UNDO;
-        break;
-    case MODE_SET_COMMIT:
-        nRequestType = SNMP_EXTENSION_SET_COMMIT;
-        break;
-    case MODE_SET_FREE:
-        nRequestType = SNMP_EXTENSION_SET_CLEANUP;
-        break;
-    default:
-        DEBUGMSG(("winExtDLL",
-                  "internal error: invalid mode %d.\n", reqinfo->mode));
-        netsnmp_assert(0);
+    if (!get_request_type(reqinfo->mode, !!ext_dll_info->pfSnmpExtensionQueryEx,
+                          &nRequestType)) {
         return SNMP_ERR_NOERROR;
     }
 
@@ -784,6 +904,30 @@ var_winExtDLL(netsnmp_mib_handler *handler,
                                     win_varbinds.list[0].name.idLength,
                                     reginfo->rootoid,
                                     reginfo->rootoid_len) < 0) {
+            DEBUGIF("winExtDLL") {
+                size_t          oid1_namelen = 0, oid2_namelen = 0, outlen1 = 0,
+                                outlen2 = 0;
+                char           *oid1_name = NULL, *oid2_name = NULL;
+                int             overflow1 = 0, overflow2 = 0;
+
+                netsnmp_static_assert(sizeof(oid) == sizeof(UINT));
+                netsnmp_sprint_realloc_objid((u_char **) & oid1_name,
+                                             &oid1_namelen, &outlen1, 1,
+                                             &overflow1, (const oid *)
+                                             win_varbinds.list[0].name.ids,
+                                             win_varbinds.list[0].name.idLength);
+                netsnmp_sprint_realloc_objid((u_char **) & oid2_name,
+                                             &oid2_namelen, &outlen2, 1,
+                                             &overflow2, reginfo->rootoid,
+                                             reginfo->rootoid_len);
+                DEBUGMSG(("winExtDLL",
+                          "extension DLL %s: replacing OID %s%s by OID %s%s.\n",
+                          ext_dll_info->dll_name,
+                          oid1_name, overflow1 ? " [TRUNCATED]" : "",
+                          oid2_name, overflow2 ? " [TRUNCATED]" : ""));
+                free(oid2_name);
+                free(oid1_name);
+            }
 
             SnmpUtilOidFree(&win_varbinds.list[0].name);
             memset(&win_varbinds.list[0].name, 0,
@@ -823,9 +967,26 @@ var_winExtDLL(netsnmp_mib_handler *handler,
 
         rc = convert_win_snmp_err(ErrorStatus);
         if (rc != SNMP_ERR_NOERROR) {
-            DEBUGMSG(("winExtDLL",
-                      "extension DLL %s: SNMP query function returned error code %lu (Windows) / %d (Net-SNMP).\n",
-                      ext_dll_info->dll_name, ErrorStatus, rc));
+            DEBUGIF("winExtDLL") {
+                size_t          oid_namelen = 0, outlen = 0;
+                char           *oid_name = NULL;
+                int             overflow = 0;
+
+                netsnmp_sprint_realloc_objid((u_char **) & oid_name,
+                                             &oid_namelen,
+                                             &outlen, 1, &overflow,
+                                             ext_dll_view_info->name,
+                                             ext_dll_view_info->name_length);
+                DEBUGMSG(("winExtDLL", "extension DLL %s: SNMP query function"
+                          " returned error code %lu (Windows) / %d (Net-SNMP)"
+                          " for request type %d, OID %s%s, ASN type %d and"
+                          " value %ld.\n",
+                          ext_dll_info->dll_name, ErrorStatus, rc, nRequestType,
+                          oid_name, overflow ? " [TRUNCATED]" : "",
+                          win_varbinds.list[0].value.asnType,
+                          win_varbinds.list[0].value.asnValue.number));
+                free(oid_name);
+            }
             netsnmp_assert(ErrorIndex == 1);
             netsnmp_request_set_error(requests, rc);
             if (rc == SNMP_NOSUCHOBJECT || rc == SNMP_NOSUCHINSTANCE
