@@ -658,6 +658,9 @@ int    best_guess;
 	newname_len = MAX_OID_LEN;
 	if (read_objid(tag, newname, &newname_len)) {	/* long name */
 	  rtp = tp = get_tree(newname, newname_len, get_tree_head());
+	} else {
+	  /* failed to parse the OID */
+	  newname_len = 0;
 	}
       }
       else {
@@ -764,7 +767,7 @@ __add_var_val_str(pdu, name, name_length, val, len, type)
       case TYPE_INTEGER:
       case TYPE_INTEGER32:
         vars->type = ASN_INTEGER;
-        vars->val.integer = (long *)malloc(sizeof(long));
+        vars->val.integer = malloc(sizeof(long));
         if (val)
             *(vars->val.integer) = strtol(val,NULL,0);
         else {
@@ -787,7 +790,7 @@ __add_var_val_str(pdu, name, name_length, val, len, type)
       case TYPE_UINTEGER:
         vars->type = ASN_UINTEGER;
 UINT:
-        vars->val.integer = (long *)malloc(sizeof(long));
+        vars->val.integer = malloc(sizeof(long));
         if (val)
             sscanf(val,"%lu",vars->val.integer);
         else {
@@ -808,7 +811,7 @@ UINT:
       case TYPE_OPAQUE:
         vars->type = ASN_OCTET_STR;
 OCT:
-        vars->val.string = (u_char *)malloc(len);
+        vars->val.string = malloc(len);
         vars->val_len = len;
         if (val && len)
             memcpy((char *)vars->val.string, val, len);
@@ -821,14 +824,18 @@ OCT:
 
       case TYPE_IPADDR:
         vars->type = ASN_IPADDRESS;
-        vars->val.integer = (long *)malloc(sizeof(long));
-        if (val)
-            *(vars->val.integer) = inet_addr(val);
-        else {
-            ret = FAILURE;
-            *(vars->val.integer) = 0;
+        {
+            in_addr_t addr;
+
+            if (val)
+                addr = inet_addr(val);
+            else {
+                ret = FAILURE;
+                addr = 0;
+            }
+            vars->val.integer = netsnmp_memdup(&addr, sizeof(addr));
+            vars->val_len = sizeof(addr);
         }
-        vars->val_len = sizeof(long);
         break;
 
       case TYPE_OBJID:
@@ -858,15 +865,8 @@ OCT:
 /* takes ss and pdu as input and updates the 'response' argument */
 /* the input 'pdu' argument will be freed */
 static int
-__send_sync_pdu(ss, pdu, response, retry_nosuch,
-	        err_str, err_num, err_ind)
-netsnmp_session *ss;
-netsnmp_pdu *pdu;
-netsnmp_pdu **response;
-int retry_nosuch;
-char *err_str;
-int *err_num;
-int *err_ind;
+__send_sync_pdu(netsnmp_session *ss, netsnmp_pdu *pdu, netsnmp_pdu **response,
+                int retry_nosuch, char *err_str, int *err_num, int *err_ind)
 {
    int status = 0;
    long command = pdu->command;
@@ -881,6 +881,7 @@ int *err_ind;
    if (ss == NULL) {
        *err_num = 0;
        *err_ind = SNMPERR_BAD_SESSION;
+       status = SNMPERR_BAD_SESSION;
        strlcpy(err_str, snmp_api_errstring(*err_ind), STR_BUF_SIZE);
        goto done;
    }
@@ -1488,6 +1489,7 @@ netsnmp_get(PyObject *self, PyObject *args)
 	    printf("error: get: unknown object ID (%s)",
 		   (tag ? tag : "<null>"));
 	  snmp_free_pdu(pdu);
+	  Py_DECREF(varbind); 
 	  goto done;
 	}
 	/* release reference when done */
@@ -1589,13 +1591,20 @@ netsnmp_get(PyObject *self, PyObject *args)
 	py_netsnmp_attr_set_string(varbind, "val", (char *) str_buf, len);
 
 	/* save in return tuple as well */
-	PyTuple_SetItem(val_tuple, varlist_ind, 
-			(len ? Py_BuildValue("s#", str_buf, len) :
-			 Py_BuildValue("")));
-
+	if ((type == SNMP_ENDOFMIBVIEW) ||
+			(type == SNMP_NOSUCHOBJECT) ||
+			(type == SNMP_NOSUCHINSTANCE)) {
+		/* Translate error to None */
+		PyTuple_SetItem(val_tuple, varlist_ind, 
+			Py_BuildValue(""));
+	} else {
+		PyTuple_SetItem(val_tuple, varlist_ind,
+			Py_BuildValue("s#", str_buf, len));
+	}
 	Py_DECREF(varbind);
       } else {
 	printf("netsnmp_get: bad varbind (%d)\n", varlist_ind);
+	Py_XDECREF(varbind); 
       }	
     }
 
@@ -1702,6 +1711,7 @@ netsnmp_getnext(PyObject *self, PyObject *args)
 	    printf("error: get: unknown object ID (%s)",
 		   (tag ? tag : "<null>"));
 	  snmp_free_pdu(pdu);
+	  Py_DECREF(varbind); 
 	  goto done;
 	}
 	/* release reference when done */
@@ -1801,13 +1811,20 @@ netsnmp_getnext(PyObject *self, PyObject *args)
 	py_netsnmp_attr_set_string(varbind, "val", (char *) str_buf, len);
 
 	/* save in return tuple as well */
-	PyTuple_SetItem(val_tuple, varlist_ind, 
-			(len ? Py_BuildValue("s#", str_buf, len) :
-			 Py_BuildValue("")));
-
+	if ((type == SNMP_ENDOFMIBVIEW) ||
+			(type == SNMP_NOSUCHOBJECT) ||
+			(type == SNMP_NOSUCHINSTANCE)) {
+		/* Translate error to None */
+		PyTuple_SetItem(val_tuple, varlist_ind, 
+			Py_BuildValue(""));
+	} else {
+		PyTuple_SetItem(val_tuple, varlist_ind,
+			Py_BuildValue("s#", str_buf, len));
+	}
 	Py_DECREF(varbind);
       } else {
 	printf("netsnmp_getnext: bad varbind (%d)\n", varlist_ind);
+	Py_XDECREF(varbind); 
       }
     }
 
@@ -1952,6 +1969,7 @@ netsnmp_walk(PyObject *self, PyObject *args)
           printf("error: walk: unknown object ID (%s)",
       	   (tag ? tag : "<null>"));
         snmp_free_pdu(pdu);
+        Py_DECREF(varbind); 
         goto done;
       }
       /* release reference when done */
@@ -2130,10 +2148,7 @@ netsnmp_walk(PyObject *self, PyObject *args)
                   /* save in return tuple as well - steals ref */
                   _PyTuple_Resize(&val_tuple, result_count+1);
                   PyTuple_SetItem(val_tuple, result_count++, 
-                                  (len ? Py_BuildValue("s#", str_buf, len) :
-                                   Py_BuildValue("")));
-            
-
+                                  Py_BuildValue("s#", str_buf, len));
               } else {
                   /* Return None for this variable. */
                   _PyTuple_Resize(&val_tuple, result_count+1);
@@ -2276,6 +2291,7 @@ netsnmp_getbulk(PyObject *self, PyObject *args)
 	    printf("error: get: unknown object ID (%s)",
 		   (tag ? tag : "<null>"));
 	  snmp_free_pdu(pdu);
+	  Py_DECREF(varbind); 
 	  goto done;
 	}
 	/* release reference when done */
@@ -2395,6 +2411,7 @@ netsnmp_getbulk(PyObject *self, PyObject *args)
 	    PyList_Append(varbinds, none); /* increments ref */
 	    /* Return None for this variable. */
 	    PyTuple_SetItem(val_tuple, varbind_ind, none); /* steals ref */
+	    Py_XDECREF(varbind); 
 	  }	
 	}
       }
@@ -2562,6 +2579,7 @@ netsnmp_set(PyObject *self, PyObject *args)
       ret = Py_BuildValue("i",0); /* fail, return False */
   } 
  done:
+  Py_XDECREF(varbind); 
   SAFE_FREE(oid_arr);
   return (ret ? ret : Py_BuildValue(""));
 }
