@@ -126,6 +126,7 @@
  *  @{
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <sys/types.h>
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -142,6 +143,9 @@
 #include <strings.h>
 #endif
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif
@@ -154,6 +158,14 @@
 
 #include <net-snmp/library/snmp_api.h>
 
+netsnmp_feature_child_of(default_store_all, libnetsnmp)
+
+netsnmp_feature_child_of(default_store_void, default_store_all)
+
+#ifndef NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID
+#endif /* NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID */
+
+
 static const char * stores [NETSNMP_DS_MAX_IDS] = { "LIB", "APP", "TOK" };
 
 typedef struct netsnmp_ds_read_config_s {
@@ -170,7 +182,9 @@ static netsnmp_ds_read_config *netsnmp_ds_configs = NULL;
 static int   netsnmp_ds_integers[NETSNMP_DS_MAX_IDS][NETSNMP_DS_MAX_SUBIDS];
 static char  netsnmp_ds_booleans[NETSNMP_DS_MAX_IDS][NETSNMP_DS_MAX_SUBIDS/8];
 static char *netsnmp_ds_strings[NETSNMP_DS_MAX_IDS][NETSNMP_DS_MAX_SUBIDS];
+#ifndef NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID
 static void *netsnmp_ds_voids[NETSNMP_DS_MAX_IDS][NETSNMP_DS_MAX_SUBIDS];
+#endif /* NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID */
 
 /*
  * Prototype definitions 
@@ -310,6 +324,7 @@ netsnmp_ds_get_string(int storeid, int which)
     return netsnmp_ds_strings[storeid][which];
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID
 int
 netsnmp_ds_set_void(int storeid, int which, void *value)
 {
@@ -336,6 +351,7 @@ netsnmp_ds_get_void(int storeid, int which)
 
     return netsnmp_ds_voids[storeid][which];
 }
+#endif /* NETSNMP_FEATURE_REMOVE_DEFAULT_STORE_VOID */
 
 int
 netsnmp_ds_parse_boolean(char *line)
