@@ -28,6 +28,6 @@ gettimeofday(struct timeval *tv, struct timezone *tz)
 
     _ftime(&timebuffer);
     tv->tv_usec = timebuffer.millitm * 1000;
-    tv->tv_sec = timebuffer.time;
+    tv->tv_sec = (long)timebuffer.time;
     return (0);
 }
