@@ -153,26 +153,31 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
     ctx = EVP_MD_CTX_create();
 #else
     ctx = malloc(sizeof(*ctx));
-    EVP_MD_CTX_init(ctx);
+    if (!EVP_MD_CTX_init(ctx))
+        return SNMPERR_GENERR;
 #endif
 #ifndef NETSNMP_DISABLE_MD5
-    if (ISTRANSFORM(hashtype, HMACMD5Auth))
-        EVP_DigestInit(ctx, EVP_md5());
-    else
+    if (ISTRANSFORM(hashtype, HMACMD5Auth)) {
+        if (!EVP_DigestInit(ctx, EVP_md5()))
+            return SNMPERR_GENERR;
+    } else
 #endif
-        if (ISTRANSFORM(hashtype, HMACSHA1Auth))
-        EVP_DigestInit(ctx, EVP_sha1());
-    else
+        if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
+            if (!EVP_DigestInit(ctx, EVP_sha1()))
+                return SNMPERR_GENERR;
+        } else
         QUITFUN(SNMPERR_GENERR, generate_Ku_quit);
 #elif NETSNMP_USE_INTERNAL_CRYPTO
 #ifndef NETSNMP_DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth)) {
-        MD5_Init(&cmd5);
+        if (!MD5_Init(&cmd5))
+            return SNMPERR_GENERR;
         cryptotype = TYPE_MD5;
     } else
 #endif
            if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
-        SHA1_Init(&csha1);
+        if (!SHA1_Init(&csha1))
+            return SNMPERR_GENERR;
         cryptotype = TYPE_SHA1;
     } else {
         return (SNMPERR_GENERR);
