@@ -19,7 +19,11 @@
 #include <net-snmp/library/large_fd_set.h>
 
 
-#if ! defined(cygwin) && defined(HAVE_WINSOCK_H)
+#if !defined(cygwin) && defined(HAVE_WINSOCK_H)
+
+#define LFD_SET(n, p)    FD_SET(n, p)
+#define LFD_CLR(n, p)    FD_CLR(n, p)
+#define LFD_ISSET(n, p)  FD_ISSET(n, p)
 
 void
 netsnmp_large_fd_setfd(SOCKET fd, netsnmp_large_fd_set * fdset)
@@ -39,7 +43,7 @@ netsnmp_large_fd_setfd(SOCKET fd, netsnmp_large_fd_set * fdset)
     if (i == fdset->lfs_set.fd_count
         && fdset->lfs_set.fd_count < fdset->lfs_setsize) {
         fdset->lfs_set.fd_count++;
-        fdset->lfs_set.fd_array[i] = (fd);
+        fdset->lfs_set.fd_array[i] = fd;
     }
 }
 
@@ -51,7 +55,7 @@ netsnmp_large_fd_clr(SOCKET fd, netsnmp_large_fd_set * fdset)
     netsnmp_assert(fd != INVALID_SOCKET);
 
     for (i = 0; i < fdset->lfs_set.fd_count; i++) {
-        if (fdset->lfs_set.fd_array[i] == (fd)) {
+        if (fdset->lfs_set.fd_array[i] == fd) {
             while (i < fdset->lfs_set.fd_count - 1) {
                 fdset->lfs_set.fd_array[i] =
                     fdset->lfs_set.fd_array[i + 1];
@@ -79,6 +83,14 @@ netsnmp_large_fd_is_set(SOCKET fd, netsnmp_large_fd_set * fdset)
 
 #else
 
+/*
+ * Recent versions of glibc trigger abort() if FD_SET(), FD_CLR() or
+ * FD_ISSET() is invoked with n >= FD_SETSIZE. Hence these replacement macros.
+ */
+#define LFD_SET(n, p)    ((p)->fds_bits[(n)/NFDBITS] |=  (1ULL << ((n) % NFDBITS)))
+#define LFD_CLR(n, p)    ((p)->fds_bits[(n)/NFDBITS] &= ~(1ULL << ((n) % NFDBITS)))
+#define LFD_ISSET(n, p)  ((p)->fds_bits[(n)/NFDBITS] &   (1ULL << ((n) % NFDBITS)))
+
 void
 netsnmp_large_fd_setfd(int fd, netsnmp_large_fd_set * fdset)
 {
@@ -87,7 +99,7 @@ netsnmp_large_fd_setfd(int fd, netsnmp_large_fd_set * fdset)
     while (fd >= (int)fdset->lfs_setsize)
         netsnmp_large_fd_set_resize(fdset, 2 * (fdset->lfs_setsize + 1));
 
-    FD_SET(fd, fdset->lfs_setptr);
+    LFD_SET(fd, fdset->lfs_setptr);
 }
 
 void
@@ -95,8 +107,8 @@ netsnmp_large_fd_clr(int fd, netsnmp_large_fd_set * fdset)
 {
     netsnmp_assert(fd >= 0);
 
-    if (fd < (int)fdset->lfs_setsize)
-        FD_CLR(fd, fdset->lfs_setptr);
+    if ((unsigned)fd < fdset->lfs_setsize)
+        LFD_CLR(fd, fdset->lfs_setptr);
 }
 
 int
@@ -104,7 +116,8 @@ netsnmp_large_fd_is_set(int fd, netsnmp_large_fd_set * fdset)
 {
     netsnmp_assert(fd >= 0);
 
-    return fd < (int)fdset->lfs_setsize && FD_ISSET(fd, fdset->lfs_setptr);
+    return ((unsigned)fd < fdset->lfs_setsize &&
+            LFD_ISSET(fd, fdset->lfs_setptr));
 }
 
 #endif
@@ -114,21 +127,56 @@ netsnmp_large_fd_set_init(netsnmp_large_fd_set * fdset, int setsize)
 {
     fdset->lfs_setsize = 0;
     fdset->lfs_setptr  = NULL;
+#if !defined(cygwin) && defined(HAVE_WINSOCK_H)
+    fdset->lfs_set.fd_count = 0;
+#endif
     netsnmp_large_fd_set_resize(fdset, setsize);
 }
 
-void
+int
+netsnmp_large_fd_set_select(int numfds, netsnmp_large_fd_set *readfds,
+                     netsnmp_large_fd_set *writefds,
+                     netsnmp_large_fd_set *exceptfds,
+                     struct timeval *timeout)
+{
+#if defined(cygwin) || !defined(HAVE_WINSOCK_H)
+    /* Bit-set representation: make sure all fds have at least size 'numfds'. */
+    if (readfds && readfds->lfs_setsize < numfds)
+        netsnmp_large_fd_set_resize(readfds, numfds);
+    if (writefds && writefds->lfs_setsize < numfds)
+        netsnmp_large_fd_set_resize(writefds, numfds);
+    if (exceptfds && exceptfds->lfs_setsize < numfds)
+        netsnmp_large_fd_set_resize(exceptfds, numfds);
+#else
+    /* Array representation: no resizing is necessary. */
+#endif
+
+    return select(numfds,
+            readfds ? readfds->lfs_setptr : NULL,
+            writefds ? writefds->lfs_setptr : NULL,
+            exceptfds ? exceptfds->lfs_setptr : NULL,
+            timeout);
+}
+
+int
 netsnmp_large_fd_set_resize(netsnmp_large_fd_set * fdset, int setsize)
 {
     int             fd_set_bytes;
 
+    if (fdset->lfs_setsize == setsize)
+        goto success;
+
     if (setsize > FD_SETSIZE) {
         fd_set_bytes = NETSNMP_FD_SET_BYTES(setsize);
-        if (fdset->lfs_setsize > FD_SETSIZE)
-            fdset->lfs_setptr = (fd_set *)realloc(fdset->lfs_setptr, fd_set_bytes);
-        else {
-            fdset->lfs_setptr = (fd_set *)malloc(fd_set_bytes);
-           *fdset->lfs_setptr = fdset->lfs_set;
+        if (fdset->lfs_setsize > FD_SETSIZE) {
+            fdset->lfs_setptr = realloc(fdset->lfs_setptr, fd_set_bytes);
+            if (!fdset->lfs_setptr)
+                goto out_of_mem;
+        } else {
+            fdset->lfs_setptr = malloc(fd_set_bytes);
+            if (!fdset->lfs_setptr)
+                goto out_of_mem;
+            *fdset->lfs_setptr = fdset->lfs_set;
         }
     } else {
         if (fdset->lfs_setsize > FD_SETSIZE) {
@@ -138,20 +186,36 @@ netsnmp_large_fd_set_resize(netsnmp_large_fd_set * fdset, int setsize)
         fdset->lfs_setptr = &fdset->lfs_set;
     }
 
-#if ! (! defined(cygwin) && defined(HAVE_WINSOCK_H))
-    {
+#if defined(cygwin) || !defined(HAVE_WINSOCK_H)
+    /*
+     * Unix: when enlarging, clear the file descriptors defined in the
+     * resized *fdset but that were not defined in the original *fdset.
+     */
+    if ( fdset->lfs_setsize == 0 && setsize == FD_SETSIZE ) {
+        /* In this case we can use the OS's FD_ZERO */
+        FD_ZERO(fdset->lfs_setptr);
+    } else {
         int             i;
 
-	/*
-	 * Unix: clear the file descriptors defined in the resized *fdset
-	 * but that were not defined in the original *fdset.
-	 */
-	for (i = fdset->lfs_setsize + 1; i < setsize; i++)
-	    FD_CLR(i, fdset->lfs_setptr);
+        for (i = fdset->lfs_setsize; i < setsize; i++)
+            LFD_CLR(i, fdset->lfs_setptr);
     }
 #endif
 
     fdset->lfs_setsize = setsize;
+#if !defined(cygwin) && defined(HAVE_WINSOCK_H)
+    if (setsize < fdset->lfs_set.fd_count)
+        fdset->lfs_set.fd_count = setsize;
+#endif
+success:
+    return 1;
+
+out_of_mem:
+    fdset->lfs_setsize = 0;
+#if !defined(cygwin) && defined(HAVE_WINSOCK_H)
+    fdset->lfs_set.fd_count = 0;
+#endif
+    return 0;
 }
 
 void
@@ -182,13 +246,13 @@ netsnmp_copy_large_fd_set_to_fd_set(fd_set * dst,
 
     *dst = *src->lfs_setptr;
 
-#if ! (! defined(cygwin) && defined(HAVE_WINSOCK_H))
+#if !(!defined(cygwin) && defined(HAVE_WINSOCK_H))
     {
         int             i;
 
-	/* Unix: clear any file descriptors defined in *dst but not in *src. */
-	for (i = src->lfs_setsize; i < FD_SETSIZE; ++i)
-	  FD_CLR(i, dst);
+        /* Unix: clear any file descriptors defined in *dst but not in *src. */
+        for (i = src->lfs_setsize; i < FD_SETSIZE; ++i)
+            FD_CLR(i, dst);
     }
 #endif
 
