@@ -536,13 +536,13 @@ run_config_handler(struct config_line *lptr,
     lptr = read_config_find_handler(lptr, token);
     if (lptr != NULL) {
         if (when == EITHER_CONFIG || lptr->config_time == when) {
+            char tmpbuf[1];
             DEBUGMSGTL(("read_config:parser",
                         "Found a parser.  Calling it: %s / %s\n", token,
                         cptr));
             /*
              * Make sure cptr is non-null
              */
-            char tmpbuf[1];
             if (!cptr) {
                 tmpbuf[0] = '\0';
                 cptr = tmpbuf;
@@ -907,7 +907,7 @@ read_config(const char *filename,
                         continue;
                     }
                     while ((entry = readdir( d )) != NULL ) {
-                        if ( entry->d_name && entry->d_name[0] != '.') {
+                        if ( entry->d_name[0] != '.') {
                             len = NAMLEN(entry);
                             if ((len > 5) && (strcmp(&(entry->d_name[len-5]),".conf") == 0)) {
                                 snprintf(fname, SNMP_MAXPATH, "%s/%s",
@@ -1885,23 +1885,32 @@ copy_word(char *from, char *to)
     return copy_nword(from, to, SPRINT_MAX_LEN);
 }                               /* copy_word */
 
-/*
- * read_config_save_octet_string(): saves an octet string as a length
- * followed by a string of hex 
+/**
+ * Stores an quoted version of the first len bytes from str into saveto.
+ *
+ * If all octets in str are in the set [[:alnum:] ] then the quotation
+ * is to enclose the string in quotation marks ("str") otherwise the
+ * quotation is to prepend the string 0x and then add the hex representation
+ * of all characters from str (0x737472)
+ *
+ * @param[in] saveto pointer to output stream, is assumed to be big enough.
+ * @param[in] str pointer of the data that is to be stored.
+ * @param[in] len length of the data that is to be stored.
+ * @return A pointer to saveto after str is added to it.
  */
 char           *
-read_config_save_octet_string(char *saveto, u_char * str, size_t len)
+read_config_save_octet_string(char *saveto, const u_char * str, size_t len)
 {
-    int             i;
-    u_char         *cp;
+    size_t          i;
+    const u_char   *cp;
 
     /*
-     * is everything easily printable 
+     * is everything easily printable
      */
-    for (i = 0, cp = str; i < (int) len && cp &&
+    for (i = 0, cp = str; i < len && cp &&
          (isalpha(*cp) || isdigit(*cp) || *cp == ' '); cp++, i++);
 
-    if (len != 0 && i == (int) len) {
+    if (len != 0 && i == len) {
         *saveto++ = '"';
         memcpy(saveto, str, len);
         saveto += len;
@@ -1911,7 +1920,7 @@ read_config_save_octet_string(char *saveto, u_char * str, size_t len)
         if (str != NULL) {
             sprintf(saveto, "0x");
             saveto += 2;
-            for (i = 0; i < (int) len; i++) {
+            for (i = 0; i < len; i++) {
                 sprintf(saveto, "%02x", str[i]);
                 saveto = saveto + 2;
             }
