@@ -436,7 +436,7 @@ netsnmp_view_get(struct vacm_viewEntry *head, const char *viewName,
     int count=0;
 
     glen = (int) strlen(viewName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     view[0] = glen;
     strcpy(view + 1, viewName);
@@ -519,7 +519,7 @@ netsnmp_view_subtree_check(struct vacm_viewEntry *head, const char *viewName,
     int             found, glen;
 
     glen = (int) strlen(viewName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return VACM_NOTINVIEW;
     view[0] = glen;
     strcpy(view + 1, viewName);
@@ -675,7 +675,7 @@ netsnmp_view_create(struct vacm_viewEntry **head, const char *viewName,
     int             cmp, cmp2, glen;
 
     glen = (int) strlen(viewName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     vp = (struct vacm_viewEntry *) calloc(1,
                                           sizeof(struct vacm_viewEntry));
@@ -765,7 +765,7 @@ vacm_getGroupEntry(int securityModel, const char *securityName)
     int             glen;
 
     glen = (int) strlen(securityName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     secname[0] = glen;
     strcpy(secname + 1, securityName);
@@ -801,7 +801,7 @@ vacm_createGroupEntry(int securityModel, const char *securityName)
     int             cmp, glen;
 
     glen = (int) strlen(securityName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     gp = (struct vacm_groupEntry *) calloc(1,
                                            sizeof(struct vacm_groupEntry));
@@ -842,6 +842,7 @@ vacm_createGroupEntry(int securityModel, const char *securityName)
     return gp;
 }
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
 void
 vacm_destroyGroupEntry(int securityModel, const char *securityName)
 {
@@ -867,6 +868,7 @@ vacm_destroyGroupEntry(int securityModel, const char *securityName)
     free(vp);
     return;
 }
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
 void
 vacm_destroyAllGroupEntries(void)
@@ -930,10 +932,10 @@ vacm_getAccessEntry(const char *groupName,
     int             glen, clen;
 
     glen = (int) strlen(groupName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     clen = (int) strlen(contextPrefix);
-    if (clen < 0 || clen >= VACM_MAX_STRING)
+    if (clen < 0 || clen > VACM_MAX_STRING)
         return NULL;
 
     group[0] = glen;
@@ -982,10 +984,10 @@ vacm_createAccessEntry(const char *groupName,
     int             cmp, glen, clen;
 
     glen = (int) strlen(groupName);
-    if (glen < 0 || glen >= VACM_MAX_STRING)
+    if (glen < 0 || glen > VACM_MAX_STRING)
         return NULL;
     clen = (int) strlen(contextPrefix);
-    if (clen < 0 || clen >= VACM_MAX_STRING)
+    if (clen < 0 || clen > VACM_MAX_STRING)
         return NULL;
     vp = (struct vacm_accessEntry *) calloc(1,
                                             sizeof(struct
@@ -1038,6 +1040,7 @@ vacm_createAccessEntry(const char *groupName,
     return vp;
 }
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
 void
 vacm_destroyAccessEntry(const char *groupName,
                         const char *contextPrefix,
@@ -1069,6 +1072,7 @@ vacm_destroyAccessEntry(const char *groupName,
     free(vp);
     return;
 }
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
 void
 vacm_destroyAllAccessEntries(void)
@@ -1142,12 +1146,14 @@ vacm_createViewEntry(const char *viewName,
                                 viewSubtreeLen);
 }
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
 void
 vacm_destroyViewEntry(const char *viewName,
                       oid * viewSubtree, size_t viewSubtreeLen)
 {
     netsnmp_view_destroy( &viewList, viewName, viewSubtree, viewSubtreeLen);
 }
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
 void
 vacm_destroyAllViewEntries(void)
